import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:jiffy/jiffy.dart';
import 'package:provider/provider.dart';
import 'package:flutter_localizations/flutter_localizations.dart';

import 'app.dart';
import 'model/accountData.dart';
import 'model/accountModel.dart';
import 'model/chatList/chatListProps.dart';
import 'model/chatList/chatProps.dart';
import 'model/files/filesProps.dart';
import 'model/holidays/holidaysProps.dart';
import 'model/message/messageProps.dart';
import 'model/timetable/timetableProps.dart';
import 'storage/base/settingsProvider.dart';
import 'theming/darkAppTheme.dart';
import 'theming/lightAppTheme.dart';
import 'view/login/login.dart';
import 'widget/placeholderView.dart';

Future<void> main() async {
  WidgetsFlutterBinding.ensureInitialized();

  ByteData data = await PlatformAssetBundle().load('assets/ca/lets-encrypt-r3.pem');
  SecurityContext.defaultContext.setTrustedCertificatesBytes(data.buffer.asUint8List());

  ErrorWidget.builder = (error) {
    return PlaceholderView(icon: Icons.phonelink_erase_rounded, text: error.toString());
  };

  runApp(
      MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (context) => SettingsProvider()),
          ChangeNotifierProvider(create: (context) => AccountModel()),

          ChangeNotifierProvider(create: (context) => TimetableProps()),
          ChangeNotifierProvider(create: (context) => ChatListProps()),
          ChangeNotifierProvider(create: (context) => ChatProps()),
          ChangeNotifierProvider(create: (context) => FilesProps()),

          ChangeNotifierProvider(create: (context) => MessageProps()),
          ChangeNotifierProvider(create: (context) => HolidaysProps()),
        ],
        child: const Main(),
      )
  );
}

class Main extends StatefulWidget {
  const Main({Key? key}) : super(key: key);

  @override
  State<Main> createState() => _MainState();
}

class _MainState extends State<Main> {
  @override
  void initState() {
    Jiffy.setLocale("de");

    SystemChrome.setPreferredOrientations([DeviceOrientation.portraitUp, DeviceOrientation.portraitDown]);

    AccountData().waitForPopulation().then((value) {
      Provider.of<AccountModel>(context, listen: false)
          .setState(value ? AccountModelState.loggedIn : AccountModelState.loggedOut);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Directionality(
      textDirection: TextDirection.ltr,
      child: Consumer<SettingsProvider>(
        builder: (context, settings, child) {
          return MaterialApp(
            debugShowCheckedModeBanner: false,
            localizationsDelegates: const [
              ...GlobalMaterialLocalizations.delegates,
              GlobalWidgetsLocalizations.delegate,
            ],
            supportedLocales: const [
              Locale('de'),
              Locale('en'),
            ],
            locale: const Locale('de'),

            title: 'Marianum Fulda',

            themeMode: settings.val().appTheme,
            theme: LightAppTheme.theme,
            darkTheme: DarkAppTheme.theme,
            home: Consumer<AccountModel>(
              builder: (context, accountModel, child) {
                switch(accountModel.state) {
                  case AccountModelState.loggedIn: return const App();
                  case AccountModelState.loggedOut: return const Login();
                  case AccountModelState.undefined: return const PlaceholderView(icon: Icons.timer, text: "Daten werden geladen");
                }
              },
            ),
          );
        },
      ),
    );
  }
}
